using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Schema;
using Microsoft.Xna.Framework;
using GarageGames.Torque.Core;
using GarageGames.Torque.Util;
using GarageGames.Torque.Sim;
using GarageGames.Torque.T2D;
using GarageGames.Torque.SceneGraph;
using GarageGames.Torque.MathUtil;
using GarageGames.Torque.Materials;



namespace StarterGame2D
{
    [TorqueXmlSchemaType]
    public class T2DFadeoutComponent : TorqueComponent, ITickObject
    {
        //======================================================
        #region Static methods, fields, constructors
        #endregion

        //======================================================
        #region Constructors
        #endregion

        //======================================================
        #region Public properties, operators, constants, and enums

        public T2DSceneObject SceneObject
        {
            get { return Owner as T2DSceneObject; }
        }

        [TorqueXmlSchemaType(DefaultValue = "1.0")]
        public float StartOpacity
        {
            get { return _startOpacity; }
            set { _startOpacity = value; }
        }

        [TorqueXmlSchemaType(DefaultValue = "0.0")]
        public float EndOpacity
        {
            get { return _endOpacity; }
            set { _endOpacity = value; }
        }

        [TorqueXmlSchemaType(DefaultValue = "1.0")]
        public float FadeSeconds
        {
            get { return _fadeSeconds; }
            set { _fadeSeconds = value; }
        }

        [TorqueXmlSchemaType(DefaultValue = "true")]
        public bool RemoveOnFadeout
        {
            get { return _removeOnFadeout; }
            set { _removeOnFadeout = value; }
        }

        [TorqueXmlSchemaType(DefaultValue = "false")]
        public bool FadeOutOnCreate
        {
            get { return _fadeOutOnCreate; }
            set { _fadeOutOnCreate = value; }
        }

        #endregion

        //======================================================
        #region Public methods

        public virtual void ProcessTick(Move move, float dt)
        {
            if (_isFadingOut)
            {
                SceneObject.VisibilityLevel -= _fadeSeconds / 1000;

                if (SceneObject.VisibilityLevel <= 0 && _removeOnFadeout == true)
                    SceneObject.MarkForDelete = true;
            }
            else if (_isFadingIn)
            {
                SceneObject.VisibilityLevel += _fadeSeconds / 1000;
            }
        }

        public virtual void InterpolateTick(float k)
        {
        }

        public override void CopyTo(TorqueComponent obj)
        {
            base.CopyTo(obj);

            T2DFadeoutComponent obj2 = obj as T2DFadeoutComponent;
            obj2._startOpacity = _startOpacity;
            obj2._endOpacity = _endOpacity;
            obj2._isFadingOut = _isFadingOut;
            obj2._isFadingIn = _isFadingIn;
            obj2._removeOnFadeout = _removeOnFadeout;
            obj2._fadeOutOnCreate = _fadeOutOnCreate;
        }

        public void FadeOut()
        {
            _startOpacity = 1.0f;
            _endOpacity = 0.0f;
            _isFadingIn = false;
            _isFadingOut = true;
        }

        public void FadeIn()
        {
            _startOpacity = 0.0f;
            _endOpacity = 0.1f;
            _isFadingOut = false;
            _isFadingIn = true;
        }

        #endregion

        //======================================================
        #region Private, protected, internal methods

        protected override bool _OnRegister(TorqueObject owner)
        {
            if (!base._OnRegister(owner) || !(owner is T2DSceneObject))
                return false;

            SimpleMaterial material = null;

            if (SceneObject is T2DStaticSprite)
                material = (SimpleMaterial)((T2DStaticSprite)SceneObject).Material;

            if (SceneObject is T2DAnimatedSprite)
                material = (SimpleMaterial)((T2DAnimatedSprite)SceneObject).AnimationData.Material;

            if (SceneObject is T2DScroller)
                material = (SimpleMaterial)((T2DScroller)SceneObject).Material;

            Assert.Fatal(material == null, "T2DFadeoutComponent must be attached to a 2D scene object with a material.");

            material.IsTranslucent = true;
            material.IsColorBlended = true;

            if (_fadeOutOnCreate)
                FadeOut();

            ProcessList.Instance.AddTickCallback(Owner, this);

            return true;
        }

        protected override void _OnUnregister()
        {
            base._OnUnregister();
        }

        protected override void _RegisterInterfaces(TorqueObject owner)
        {
            base._RegisterInterfaces(owner);
        }


        #endregion

        //======================================================
        #region Private, protected, internal fields

        float _startOpacity = 1.0f;
        float _endOpacity = 0.0f;
        float _fadeSeconds = 1.0f;
        bool _isFadingOut = false;
        bool _isFadingIn = false;
        bool _removeOnFadeout = false;
        bool _fadeOutOnCreate = false;

        #endregion
    }
}
