using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace GarageGames.Torque
{
    public class ManagedTexture2D : Texture2D
    {
        // Summary:
        //     Creates an uninitialized Texture2D resource of the specified dimensions.
        //     To initialize a Texture2D from an existing file, see the static method Framework.Content.ContentManager.Load``1(System.String)
        //     or Texture2D.FromFile.
        //
        // Parameters:
        //   graphicsDevice:
        //     The GraphicsDevice used to display the texture.
        //
        //   width:
        //     The width of the texture, in pixels. This value must be a power of two if
        //     the GraphicsDeviceCapabilities.TextureCaps.RequiresPower2 property of graphicsDevice
        //     is true. If this value is 0, a value of 1 is used.
        //
        //   height:
        //     The height of the texture, in pixels. This value must be a power of two if
        //     the GraphicsDeviceCapabilities.TextureCaps.RequiresPower2 property of graphicsDevice
        //     is true. If this value is 0, a value of 1 is used.
        //
        //   numberLevels:
        //     The number of downsampled surfaces to create when preprocessing the texture.
        //     These smaller versions of the texture, known as mipmap levels, are used when
        //     the texture is minified to fit a smaller area than the original texture size.
        //     The chain of downsampled surfaces associated with a texture is sometimes
        //     called a mipmap chain.If numberLevels is zero, all texture sublevels down
        //     to 11 pixels will be generated for hardware that supports mipmapped textures.
        //     Use Texture.LevelCount to see the number of levels generated.
        //
        //   usage:
        //     Options identifying the behaviors of this texture resource.
        //
        //   format:
        //     A SurfaceFormat value specifying the requested pixel format for the texture.
        //     The returned texture may be of a different format if the device does not
        //     support the requested format. Applications should check the format of the
        //     returned texture to ensure that it matches the requested format.
        public ManagedTexture2D(GraphicsDevice graphicsDevice, int width, int height, uint[] data)
            : base(graphicsDevice, width, height, 1, TextureUsage.None, SurfaceFormat.Color)
        {
            SetData<uint>(data);
            ManagedData = data;
        }

        /// <summary>Gets data managed by this instance.  Modifications to this data do not affect the base texture.</summary>
        public readonly uint[] ManagedData;

        /// <summary>
        /// Creates a new ManagedTexture2D from the given Texture2D.
        /// </summary>
        public static ManagedTexture2D FromTexture2D(Texture2D instance)
        {
            int w = instance.Width;
            int h = instance.Height;
            uint[] data = new uint[w * h];
            instance.GetData<uint>(data);
            ManagedTexture2D result = new ManagedTexture2D(instance.GraphicsDevice, w, h, data);
            return result;
        }
    }
}