using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

using GarageGames.Torque.Core;
using GarageGames.Torque.Util;
using GarageGames.Torque.Sim;
using GarageGames.Torque.T2D;
using GarageGames.Torque.SceneGraph;
using GarageGames.Torque.MathUtil;
using GarageGames.Torque.Platform;

namespace GarageGames.Torque
{
    [TorqueXmlSchemaType]
    public class AnimatedCharacterComponent : TorqueComponent, ITickObject
    {
        //======================================================
        #region Static methods, fields, constructors
        #endregion

        //======================================================
        #region Constructors
        #endregion

        //======================================================
        #region Public properties, operators, constants, and enums

        public T2DSceneObject SceneObject
        {
            get { return Owner as T2DSceneObject; }
        }

        public T2DAnimatedSprite AnimatedSprite
        {
            get { return Owner as T2DAnimatedSprite; }
        }

        public T2DAnimationData AnimNorth
        {
            get { return _animNorth; }
            set { _animNorth = value; }
        }
        public T2DAnimationData AnimNorthWest
        {
            get { return _animNorthWest; }
            set { _animNorthWest = value; }
        }
        public T2DAnimationData AnimWest
        {
            get { return _animWest; }
            set { _animWest = value; }
        }
        public T2DAnimationData AnimSouthWest
        {
            get { return _animSouthWest; }
            set { _animSouthWest = value; }
        }
        public T2DAnimationData AnimSouth
        {
            get { return _animSouth; }
            set { _animSouth = value; }
        }
        public T2DAnimationData AnimSouthEast
        {
            get { return _animSouthEast; }
            set { _animSouthEast = value; }
        }
        public T2DAnimationData AnimEast
        {
            get { return _animEast; }
            set { _animEast = value; }
        }
        public T2DAnimationData AnimNorthEast
        {
            get { return _animNorthEast; }
            set { _animNorthEast = value; }
        }

        #endregion

        //======================================================
        #region Public methods

        public void PlayMyAnimation(T2DAnimationData animationData)
        {
            if (AnimatedSprite.AnimationData != animationData || !AnimatedSprite.IsAnimationPlaying)
            {
                AnimatedSprite.PlayAnimation(animationData);
                AnimatedSprite.PlayAnimation();
            }
        }

        public virtual void ProcessTick(Move move, float dt)
        {

            if (move != null)
            {
                if ((move.Sticks[0].X > 0) && (move.Sticks[0].Y > 0))
                {
                    PlayMyAnimation(_animNorthEast);
                }
                else if ((move.Sticks[0].X > 0) && (move.Sticks[0].Y < 0))
                {
                    PlayMyAnimation(_animSouthEast);
                }
                else if ((move.Sticks[0].X < 0) && (move.Sticks[0].Y > 0))
                {
                    PlayMyAnimation(_animNorthWest);
                }
                else if ((move.Sticks[0].X < 0) && (move.Sticks[0].Y < 0))
                {
                    PlayMyAnimation(_animSouthWest);
                }
                else if (move.Sticks[0].X > 0)
                {
                    PlayMyAnimation(_animEast);
                }
                else if (move.Sticks[0].X < 0)
                {
                    PlayMyAnimation(_animWest);
                }
                else if (move.Sticks[0].Y > 0)
                {
                    PlayMyAnimation(_animNorth);
                }
                else if (move.Sticks[0].Y < 0)
                {
                    PlayMyAnimation(_animSouth);
                }
                else
                {
                    AnimatedSprite.PauseAnimation();
                }


                // set our test object's Velocity based on stick/keyboard input
                _sceneObject.Physics.VelocityX = move.Sticks[0].X * 25.0f;
                _sceneObject.Physics.VelocityY = -move.Sticks[0].Y * 25.0f;
            }
        }


        public virtual void InterpolateTick(float k)
        {
            // todo: interpolate between ticks as needed here
        }

        public override void CopyTo(TorqueComponent obj)
        {
            base.CopyTo(obj);
            AnimatedCharacterComponent obj2 = obj as AnimatedCharacterComponent;
            obj2._animNorth = _animNorth;
            obj2._animNorthWest = _animNorthWest;
            obj2._animWest = _animWest;
            obj2._animSouthWest = _animSouthWest;
            obj2._animSouth = _animSouth;
            obj2._animSouthEast = _animSouthEast;
            obj2._animEast = _animEast;
            obj2._animNorthEast = _animNorthEast;
        }

        #endregion

        //======================================================
        #region Private, protected, internal methods

        protected override bool _OnRegister(TorqueObject owner)
        {
            if (!base._OnRegister(owner) || !(owner is T2DSceneObject))
                return false;

            // retain a reference to this component's owner object
            _sceneObject = owner as T2DSceneObject;
            _SetupInputMap(_sceneObject, 0, "gamepad0", "keyboard");

            // tell the process list to notifiy us with ProcessTick and InterpolateTick events
            ProcessList.Instance.AddTickCallback(Owner, this);

            return true;
        }

        private void _SetupInputMap(TorqueObject player, int playerIndex, String gamePad, String keyboard)
        {
            // Set player as the controllable object
            PlayerManager.Instance.GetPlayer(playerIndex).ControlObject = player;

            // Get input map for this player and configure it
            InputMap inputMap = PlayerManager.Instance.GetPlayer(playerIndex).InputMap;

            int gamepadId = InputManager.Instance.FindDevice(gamePad);
            if (gamepadId >= 0)
            {
                inputMap.BindMove(gamepadId, (int)XGamePadDevice.GamePadObjects.LeftThumbX, MoveMapTypes.StickAnalogHorizontal, 0);
                inputMap.BindMove(gamepadId, (int)XGamePadDevice.GamePadObjects.LeftThumbY, MoveMapTypes.StickAnalogVertical, 0);
                //inputMap.BindAction(gamepadId, (int)XGamePadDevice.GamePadObjects.Back, _OnBackButton);
            }

            // keyboard controls
            int keyboardId = InputManager.Instance.FindDevice(keyboard);
            if (keyboardId >= 0)
            {
                inputMap.BindMove(keyboardId, (int)Keys.Right, MoveMapTypes.StickDigitalRight, 0);
                inputMap.BindMove(keyboardId, (int)Keys.Left, MoveMapTypes.StickDigitalLeft, 0);
                inputMap.BindMove(keyboardId, (int)Keys.Up, MoveMapTypes.StickDigitalUp, 0);
                inputMap.BindMove(keyboardId, (int)Keys.Down, MoveMapTypes.StickDigitalDown, 0);
                // WASD
                inputMap.BindMove(keyboardId, (int)Keys.D, MoveMapTypes.StickDigitalRight, 0);
                inputMap.BindMove(keyboardId, (int)Keys.A, MoveMapTypes.StickDigitalLeft, 0);
                inputMap.BindMove(keyboardId, (int)Keys.W, MoveMapTypes.StickDigitalUp, 0);
                inputMap.BindMove(keyboardId, (int)Keys.S, MoveMapTypes.StickDigitalDown, 0);
            }
        }

        protected override void _OnUnregister()
        {
            // todo: perform de-initialization for the component

            base._OnUnregister();
        }

        protected override void _RegisterInterfaces(TorqueObject owner)
        {
            base._RegisterInterfaces(owner);

            // todo: register interfaces to be accessed by other components
            // E.g.,
            // Owner.RegisterCachedInterface("float", "interface name", this, _ourInterface);
        }


        #endregion

        //======================================================
        #region Private, protected, internal fields
        
        T2DSceneObject _sceneObject;
        T2DAnimationData _animNorth;
        T2DAnimationData _animNorthWest;
        T2DAnimationData _animWest;
        T2DAnimationData _animSouthWest;
        T2DAnimationData _animSouth;
        T2DAnimationData _animSouthEast;
        T2DAnimationData _animEast;
        T2DAnimationData _animNorthEast;

        #endregion
    }
}
